@@ -6,7 +6,7 @@ gssanova <- function(formula,family,type=NULL,data=list(),weights,
                      skip.iter=FALSE)
 {
     if (!(family%in%c("binomial","poisson","Gamma","inverse.gaussian","nbinomial",
-                      "weibull","lognorm","loglogis")))
+                      "polr","weibull","lognorm","loglogis")))
         stop("gss error in gssanova: family not implemented")
     if (is.null(alpha)) {
         alpha <- 1.4
@@ -86,7 +86,16 @@ gssanova <- function(formula,family,type=NULL,data=list(),weights,
     if (qr(s)$rank<dim(s)[2])
         stop("gss error in gssanova: unpenalized terms are linearly dependent")
     ## Prepare the data
-    y <- model.response(mf,"numeric")
+    if (family=="polr") {
+        y <- model.response(mf)
+        if (!is.factor(y))
+            stop("gss error in gssanova1: need factor response for polr family")
+        lvls <- levels(y)
+        if (nlvl <- length(lvls)<3)
+            stop("gss error in gssanova1: need at least 3 levels to fit polr family")
+        y <- outer(y,lvls,"==")
+    }
+    else y <- model.response(mf,"numeric")
     offset <- model.offset(mf)
     if (!is.null(offset)) {
         term$labels <- c(term$labels,"offset")
@@ -159,10 +168,12 @@ sspngreg <- function(family,s,r,q,y,wt,offset,alpha,nu,random)
         z <- ngreg(dc,family,cbind(s,10^theta*r),q.wk,y,wt,offset,nu.wk,alpha.wk)
         assign("dc",z$dc,inherits=TRUE)
         assign("fit",z[c(1:3,5:10)],inherits=TRUE)
+        if (family=="polr") assign("nu",z$nu,inherits=TRUE)
         z$score
     }
     cv.wk <- function(lambda) cv.scale*cv(lambda)+cv.shift
     ## initialization
+    dc <- rep(0,nn)
     tmp <- sum(r^2)
     if (is.null(s)) theta <- 0
     else theta <- log10(sum(s^2)/nnull/tmp*nxi) / 2
@@ -181,10 +192,18 @@ sspngreg <- function(family,s,r,q,y,wt,offset,alpha,nu,random)
                       loglogis=mkdata.loglogis(y,eta,wt,offset,nu))
         nu[[1]] <- wk$nu[[1]]
     }
+    if (family=="polr") {
+        if (is.null(wt)) P <- apply(y,2,sum)
+        else P <- apply(y*wt,2,sum)
+        P <- P/sum(P)
+        P <- cumsum(P)
+        nnu <- length(P)-2
+        dc[1] <- qlogis(P[1])
+        nu[[1]] <- diff(qlogis(P[-(nnu+2)]))
+    }
     ## lambda search
-    dc <- rep(0,nn)
     fit <- NULL
-    la <- log.la0 - 1
+    la <- log.la0
     if (nu[[2]]) la <- c(la, log(nu[[1]]))
     if (!is.null(random)) la <- c(la,random$init)
     if (length(la)-1) {
@@ -213,11 +232,14 @@ sspngreg <- function(family,s,r,q,y,wt,offset,alpha,nu,random)
         }
     }
     else {
+        mn0 <- log.la0-6
+        mx0 <- log.la0+6
         repeat {
-            mn <- la-2
-            mx <- la+1
+            mn <- max(la-1,mn0)
+            mx <- min(la+1,mx0)
             zz <- nlm0(cv,c(mn,mx))
-            if (min(zz$est-mn,mx-zz$est)>=1e-3) break
+            if ((min(zz$est-mn,mx-zz$est)>=1e-1)||
+                (min(zz$est-mn0,mx0-zz$est)<1e-1)) break
             else la <- zz$est
         }
     }
@@ -227,7 +249,7 @@ sspngreg <- function(family,s,r,q,y,wt,offset,alpha,nu,random)
         nu.wk <- exp(zz$est[2])
         zz$est <- zz$est[-2]
     }
-    else nu.wk <- NULL
+    else nu.wk <- nu[[1]]
     if (is.null(random)) q.wk <- 10^theta*q
     else {
         q.wk <- matrix(0,nxiz,nxiz)
@@ -300,6 +322,7 @@ mspngreg <- function(family,s,r,id.basis,y,wt,offset,alpha,nu,random,skip.iter)
         z <- ngreg(dc,family,cbind(s,r.wk0),q.wk,y,wt,offset,nu.wk,alpha.wk)
         assign("dc",z$dc,inherits=TRUE)
         assign("fit",z[c(1:3,5:10)],inherits=TRUE)
+        if (family=="polr") assign("nu",z$nu,inherits=TRUE)
         z$score
     }
     cv.wk <- function(theta) cv.scale*cv(theta)+cv.shift
@@ -311,7 +334,7 @@ mspngreg <- function(family,s,r,id.basis,y,wt,offset,alpha,nu,random,skip.iter)
     }
     ## theta adjustment
     z <- sspngreg(family,s,r.wk,r.wk[id.basis,],y,wt,offset,alpha,nu,random)
-    if (nu[[2]]) nu[[1]] <- z$nu
+    if (nu[[2]]|(family=="polr")) nu[[1]] <- z$nu
     theta <- theta + z$theta
     r.wk <- 0
     for (i in 1:nq) {
@@ -322,7 +345,7 @@ mspngreg <- function(family,s,r,id.basis,y,wt,offset,alpha,nu,random,skip.iter)
     log.th0 <- theta-log.la0
     ## lambda search
     z <- sspngreg(family,s,r.wk,r.wk[id.basis,],y,wt,offset,alpha,nu,random)
-    if (nu[[2]]) nu[[1]] <- z$nu
+    if (nu[[2]]|(family=="polr")) nu[[1]] <- z$nu
     nlambda <- z$nlambda
     log.th0 <- log.th0 + z$lambda
     theta <- theta + z$theta
@@ -336,6 +359,15 @@ mspngreg <- function(family,s,r,id.basis,y,wt,offset,alpha,nu,random,skip.iter)
     dc <- rep(0,nn)
     fit <- NULL
     theta.old <- theta
+    if (family=="polr") {
+        if (is.null(wt)) P <- apply(y,2,sum)
+        else P <- apply(y*wt,2,sum)
+        P <- P/sum(P)
+        P <- cumsum(P)
+        nnu <- length(P)-2
+        dc[1] <- qlogis(P[1])
+        nu[[1]] <- diff(qlogis(P[-(nnu+2)]))
+    }
     if (nu[[2]]) theta <- c(theta, log(nu[[1]]))
     if (!is.null(random)) theta <- c(theta,z$zeta)
     counter <- 0
@@ -370,7 +402,7 @@ mspngreg <- function(family,s,r,id.basis,y,wt,offset,alpha,nu,random,skip.iter)
         nu.wk <- exp(zz$est[nq+1])
         zz$est <- zz$est[-(nq+1)]
     }
-    else nu.wk <- NULL
+    else nu.wk <- nu[[1]]
     r.wk <- 0
     for (i in 1:nq) {
         r.wk <- r.wk + 10^zz$est[i]*r[,,i]
@@ -403,13 +435,12 @@ ngreg <- function(dc,family,sr,q,y,wt,offset,nu,alpha)
     nnull <- nn - nxi
     ## initialization
     cc <- dc[nnull+(1:nxi)]
-    eta <- sr%*%dc
+    eta <- as.vector(sr%*%dc)
     if (!is.null(offset)) eta <- eta + offset
     if ((family=="nbinomial")&is.vector(y)) y <- cbind(y,nu[[1]])
-    iter <- 0
-    flag <- 0
     dev <- switch(family,
                   binomial=dev.resid.binomial(y,eta,wt),
+                  polr=dev.resid.polr(y,eta,wt,nu[[1]]),
                   nbinomial=dev.resid.nbinomial(y,eta,wt),
                   poisson=dev.resid.poisson(y,eta,wt),
                   Gamma=dev.resid.Gamma(y,eta,wt),
@@ -419,24 +450,72 @@ ngreg <- function(dc,family,sr,q,y,wt,offset,nu,alpha)
                   loglogis=dev0.resid.loglogis(y,eta,wt,nu[[1]]))
     dev <- sum(dev) + t(cc)%*%q%*%cc
     ## Newton iteration
+    dc.new <- eta.new <- NULL
+    dev.line <- function(x) {
+        assign("dc.new",dc+c(x)*dc.diff,inherits=TRUE)
+        cc <- dc.new[nnull+(1:nxi)]
+        eta.wk <- as.vector(sr%*%dc.new)
+        if (!is.null(offset)) eta.wk <- eta.wk + offset
+        assign("eta.new",eta.wk,inherits=TRUE)
+        dev.wk <- switch(family,
+                         binomial=dev.resid.binomial(y,eta.new,wt),
+                         nbinomial=dev.resid.nbinomial(y,eta.new,wt),
+                         polr=dev.resid.polr(y,eta.new,wt,nu[[1]]),
+                         poisson=dev.resid.poisson(y,eta.new,wt),
+                         Gamma=dev.resid.Gamma(y,eta.new,wt),
+                         inverse.gaussian=dev.resid.inverse.gaussian(y,eta.new,wt),
+                         weibull=dev.resid.weibull(y,eta.new,wt,nu[[1]]),
+                         lognorm=dev0.resid.lognorm(y,eta.new,wt,nu[[1]]),
+                         loglogis=dev0.resid.loglogis(y,eta.new,wt,nu[[1]]))
+        sum(dev.wk) + t(cc)%*%q%*%cc
+    }
+    iter <- 0
+    flag <- 0
+    flag2 <- 0
     repeat {
         iter <- iter+1
         dat <- switch(family,
                       binomial=mkdata.binomial(y,eta,wt,offset),
                       nbinomial=mkdata.nbinomial(y,eta,wt,offset,nu),
+                      polr=mkdata.polr(y,eta,wt,offset,nu[[1]]),
                       poisson=mkdata.poisson(y,eta,wt,offset),
                       Gamma=mkdata.Gamma(y,eta,wt,offset),
                       inverse.gaussian=mkdata.inverse.gaussian(y,eta,wt,offset),
                       weibull=mkdata.weibull(y,eta,wt,offset,nu),
                       lognorm=mkdata.lognorm(y,eta,wt,offset,nu),
                       loglogis=mkdata.loglogis(y,eta,wt,offset,nu))
+        if (family=="polr") nu[[1]] <- dat$nu
         ## weighted least squares fit
+        mumax <- 2*max(abs(t(sr)%*%dat$u+c(rep(0,nnull),q%*%dc[nnull+(1:nxi)])))
         w <- as.vector(sqrt(dat$wt))
         ywk <- w*dat$ywk
         srwk <- w*sr
         if (!is.finite(sum(w,ywk,srwk))) {
             if (flag) stop("gss error in gssanova: Newton iteration diverges")
+            dc <- rep(0,nn)
             eta <- rep(0,nobs)
+            if (family=="polr") {
+                if (is.null(wt)) P <- apply(y,2,sum)
+                else P <- apply(y*wt,2,sum)
+                P <- P/sum(P)
+                P <- cumsum(P)
+                nnu <- length(P)-2
+                dc[1] <- qlogis(P[1])
+                nu[[1]] <- diff(qlogis(P[-(nnu+2)]))
+                eta <- as.vector(sr%*%dc)
+            }
+            if (!is.null(offset)) eta <- eta + offset
+            dev <- switch(family,
+                          binomial=dev.resid.binomial(y,eta,wt),
+                          nbinomial=dev.resid.nbinomial(y,eta,wt),
+                          polr=dev.resid.polr(y,eta,wt,nu[[1]]),
+                          poisson=dev.resid.poisson(y,eta,wt),
+                          Gamma=dev.resid.Gamma(y,eta,wt),
+                          inverse.gaussian=dev.resid.inverse.gaussian(y,eta,wt),
+                          weibull=dev.resid.weibull(y,eta,wt,nu[[1]]),
+                          lognorm=dev0.resid.lognorm(y,eta,wt,nu[[1]]),
+                          loglogis=dev0.resid.loglogis(y,eta,wt,nu[[1]]))
+            dev <- sum(dev)
             iter <- 0
             flag <- 1
             next
@@ -452,31 +531,47 @@ ngreg <- function(dc,family,sr,q,y,wt,offset,nu,alpha)
                       double(max(nobs,nn)), integer(1), integer(1),
                       PACKAGE="gss")["dc"]
         dc.diff <- z$dc-dc
-        adj <- 0
         repeat {
-            dc.new <- dc + dc.diff
-            cc <- dc.new[nnull+(1:nxi)]
-            eta.new <- as.vector(sr%*%dc.new)
-            if (!is.null(offset)) eta.new <- eta.new + offset
-            dev.new <- switch(family,
-                              binomial=dev.resid.binomial(y,eta.new,wt),
-                              nbinomial=dev.resid.nbinomial(y,eta.new,wt),
-                              poisson=dev.resid.poisson(y,eta.new,wt),
-                              Gamma=dev.resid.Gamma(y,eta.new,wt),
-                              inverse.gaussian=dev.resid.inverse.gaussian(y,eta.new,wt),
-                              weibull=dev.resid.weibull(y,eta.new,wt,nu[[1]]),
-                              lognorm=dev0.resid.lognorm(y,eta.new,wt,nu[[1]]),
-                              loglogis=dev0.resid.loglogis(y,eta.new,wt,nu[[1]]))
-            dev.new <- sum(dev.new) + t(cc)%*%q%*%cc
-            if (!is.finite(dev.new)) dev.new <- Inf
-            if (dev.new-dev<(1+abs(dev))*1e-1) break
-            adj <- 1
-            dc.diff <- dc.diff/2
+            dev.new <- dev.line(1)
+            if (!is.finite(dev.new)) {
+                dc.diff <- dc.diff/2
+                next
+            }
+            if (!flag2) {
+                if (dev.new-dev<1e-7*(1+abs(dev))) break
+            }
+            zz <- nlm0(dev.line,c(0,1),1e-3)
+            dev.new <- dev.line(zz$est)
+            break
         }
+        disc0 <- max((mumax/(1+eta))^2,abs(eta.new-eta)/(1+eta))
         disc <- sum(dat$wt*((eta-eta.new)/(1+abs(eta)))^2)/sum(dat$wt)
         if (!is.finite(disc)) {
             if (flag) stop("gss error in gssanova: Newton iteration diverges")
+            dc <- rep(0,nn)
             eta <- rep(0,nobs)
+            if (family=="polr") {
+                if (is.null(wt)) P <- apply(y,2,sum)
+                else P <- apply(y*wt,2,sum)
+                P <- P/sum(P)
+                P <- cumsum(P)
+                nnu <- length(P)-2
+                dc[1] <- qlogis(P[1])
+                nu[[1]] <- diff(qlogis(P[-(nnu+2)]))
+                eta <- as.vector(sr%*%dc)
+            }
+            if (!is.null(offset)) eta <- eta + offset
+            dev <- switch(family,
+                          binomial=dev.resid.binomial(y,eta,wt),
+                          nbinomial=dev.resid.nbinomial(y,eta,wt),
+                          polr=dev.resid.polr(y,eta,wt,nu[[1]]),
+                          poisson=dev.resid.poisson(y,eta,wt),
+                          Gamma=dev.resid.Gamma(y,eta,wt),
+                          inverse.gaussian=dev.resid.inverse.gaussian(y,eta,wt),
+                          weibull=dev.resid.weibull(y,eta,wt,nu[[1]]),
+                          lognorm=dev0.resid.lognorm(y,eta,wt,nu[[1]]),
+                          loglogis=dev0.resid.loglogis(y,eta,wt,nu[[1]]))
+            dev <- sum(dev)
             iter <- 0
             flag <- 1
             next
@@ -484,29 +579,28 @@ ngreg <- function(dc,family,sr,q,y,wt,offset,nu,alpha)
         dc <- dc.new
         eta <- eta.new
         dev <- dev.new
-        if (adj) next
-        if (disc<1e-7) break
+        if (min(disc0,disc)<1e-7) break
         if (iter<=30) next
-        if (!flag) {
-            eta <- rep(0,nobs)
+        if (!flag2) {
+            flag2 <- 1
             iter <- 0
-            flag <- 1
-        }
-        else {
-            warning("gss warning in gssanova: Newton iteration fails to converge")
-            break
+            next
         }
+        warning("gss warning in gssanova: Newton iteration fails to converge")
+        break
     }
     ## calculate cv
     dat <- switch(family,
                   binomial=mkdata.binomial(y,eta,wt,offset),
                   nbinomial=mkdata.nbinomial(y,eta,wt,offset,nu),
+                  polr=mkdata.polr(y,eta,wt,offset,nu[[1]]),
                   poisson=mkdata.poisson(y,eta,wt,offset),
                   Gamma=mkdata.Gamma(y,eta,wt,offset),
                   inverse.gaussian=mkdata.inverse.gaussian(y,eta,wt,offset),
                   weibull=mkdata.weibull(y,eta,wt,offset,nu),
                   lognorm=mkdata.lognorm(y,eta,wt,offset,nu),
                   loglogis=mkdata.loglogis(y,eta,wt,offset,nu))
+    if (family=="polr") nu[[1]] <- dat$nu
     ## weighted least squares fit
     w <- as.vector(sqrt(dat$wt))
     ywk <- w*dat$ywk
@@ -527,8 +621,9 @@ ngreg <- function(dc,family,sr,q,y,wt,offset,nu,alpha)
                  Gamma=cv.Gamma(y,eta,wt,z$hat[1:nobs],z$hat[nobs+1],alpha),
                  inverse.gaussian=cv.inverse.gaussian(y,eta,wt,z$hat[1:nobs],z$hat[nobs+1],alpha),
                  nbinomial=cv.nbinomial(y,eta,wt,z$hat[1:nobs],alpha),
+                 polr=cv.polr(y,eta,wt,z$hat[1:nobs],nu[[1]],alpha),
                  weibull=cv.weibull(y,eta,wt,z$hat[1:nobs],nu[[1]],alpha),
                  lognorm=cv.lognorm(y,eta,wt,z$hat[1:nobs],nu[[1]],alpha),
                  loglogis=cv.loglogis(y,eta,wt,z$hat[1:nobs],nu[[1]],alpha))
-    c(z,cv,list(eta=eta))
+    c(z,cv,list(eta=eta,nu=nu))
 }
